from typing import Union, Optional
import logging
from time import sleep
import requests
from threading import Thread, Event
from pytils.configurator import config_var_with_default

class MessageBuffer:

    def __init__(self, max_size: int = None):
        self._buffer = ''
        self._max_size = max_size

    def write(self, message: str):
        self._buffer = ''.join([self._buffer, message])[:self._max_size]

    def read(self, count: int):
        result = ''
        result, self._buffer = self._buffer[:count], self._buffer[count:]
        return result

FLUSH_INTERVAL = 5
API_HOST = 'api.telegram.org'
MAX_MESSAGE_SIZE = 4000
MAX_BUFFER_SIZE = 10**16

class TelegramLoggingHandler(logging.Handler):

    def __init__(self,
                 bot_token: str,
                 channel: Union[str, int],
                 message_thread_id: Optional[int] = None,  # Type declared here
                 level=logging.NOTSET):
        super().__init__(level)
        self._url = self._format_url(bot_token, channel, message_thread_id)
        self._buffer = MessageBuffer(MAX_BUFFER_SIZE)
        self._stop_event = Event()  # Event for stopping the thread
        self._writer_thread = None
        self._start_writer_thread()

    @staticmethod
    def _format_url(bot_token: str, channel: Union[str, int], message_thread_id: Optional[int] = None):
        formatted_channel = str(channel)
        if message_thread_id is not None:
            return f'https://{API_HOST}/bot{bot_token}/sendMessage?chat_id={formatted_channel}&message_thread_id={message_thread_id}'
        return f'https://{API_HOST}/bot{bot_token}/sendMessage?chat_id={formatted_channel}'

    def write(self, message):
        try:
            response = requests.post(self._url, data={'text': message}, timeout=5)  # Increased timeout
        except Exception as e:
            pass

    def emit(self, record: logging.LogRecord) -> None:
        message = self.format(record)
        self._buffer.write(f'{message}\n')

    def close(self):
        # Signal the thread to stop
        self._stop_event.set()
        # Wait for the thread to finish
        self._writer_thread.join()

    def _write_manager(self):
        while not self._stop_event.is_set():
            sleep(FLUSH_INTERVAL)
            message = self._buffer.read(MAX_MESSAGE_SIZE)
            if message:
                self.write(message)

    def _start_writer_thread(self):
        self._writer_thread = Thread(target=self._write_manager)
        self._writer_thread.daemon = True
        self._writer_thread.start()



import pandas
from pandas import ExcelWriter


def create_excel_table(dataframe, workbook, sheet_name='Sheet1'):
    """
    Create an Excel table from a pandas dataframe and write it to xlsxwriter workbook.

    Parameters:
    dataframe (pandas.DataFrame): The dataframe to be written to the Excel file.
    workbook (xlsxwriter.workbook.Workbook): The xlsxwriter workbook to write to.
    sheet_name (str): The name of the Excel sheet. Default is 'Sheet1'.

    Returns:
    xlsxwriter.workbook.Workbook: The updated xlsxwriter workbook.
    """
    # Ensure the index has a name
    if not dataframe.index.name:
        dataframe.index.name = 'Index'

    # Reset index to allow inserting a user defined header
    if dataframe.index.name not in dataframe.columns:
        dataframe = dataframe.reset_index()

    # Write the dataframe to the Excel workbook
    dataframe.to_excel(workbook, sheet_name=sheet_name, startrow=1, header=False, index=False)

    # Get the xlsxwriter workbook and worksheet objects
    worksheet = workbook.sheets[sheet_name]

    # Get the dimensions of the dataframe
    (max_row, max_col) = dataframe.shape

    # Create a list of column headers, to use in add_table()
    column_settings = [{'header': column} for column in dataframe.columns]

    # Add the Excel table structure. Pandas will add the data.
    worksheet.add_table(0, 0, max_row, max_col - 1, {'columns': column_settings, 'name': sheet_name})

    # Make the columns wider for clarity
    worksheet.set_column(0, max_col - 1, 12)

    return workbook
